{----------------------------------------------------------------------}
{ Component:	TbspRunAppBtn											}
{ Version:		1.0														}
{ Purpose:		Speebutton with possibility to start an application 	}
{				with click on button									}
{ Properties:															}
{	 ExecuteBeforeOnClick:	True: starts application before				}
{							handling the OnClick event					}
{	 ExecuteFile:			True: starts the application				}
{							Can be set in the OnClick-event				}
{							In this case ExecuteBeforeOnClick must		}
{							be FALSE									}
{	 UseFileIcon:			True: Uses icon associated with FileName	}
{	 FileName:				Path and Name of Application/File			}
{	 FileParams:			Parameters of App to start					}
{	 WorkingDirectory:		Working directory of app to start			}
{	 UseSmallIcon:			Small or large icons (if UseFileIcon = True	}
{	 FileWindowState:		Windowstate of app to start					}
{							(fwNormal, fwMinimized or fwMaximized)		}
{	 RunOnce: 				True: app will be started once				}
{							If button is clicked again, the app will be	}
{							brought to foreground and set active		}
{																		}
{ Known problems:														}
{	None at the moment													}
{	Please report any problems, bugfixes,... to me						}
{																		}
{ Licencing,...														}
{	This component is absolutely freeware								}
{	But for that reason you also use it at your own risk!				}
{																		}
{  2001 by bsp ag, basel, switzerland									}
{	written by Remy Schnell												}
{	eMail: remy.schnell@bsp-brainware.com								}
{----------------------------------------------------------------------}
unit bspAppExecBtn;

interface

uses Classes, Buttons, graphics;

Type
  TFileWindowState = (fwNormal, fwMinimized, fwMaximized);
  TbspRunAppBtn = Class(TSpeedButton)
  private
	 FExecuteFirst: boolean;
	 FExecuteApp: boolean;
	 FAppGlyph: boolean;
	 FAppName: string;
	 FAppParams: string;
	 FWorkDir: string;
	 FSmallIcon: boolean;
	 FAppWindowState: TFileWindowState;
	 FRunOnce: boolean;
	 // internal use only
	 FFileHandle: integer;
	 FThreadIdentifier: Longint;
	 procedure SetExecuteFirst(AValue: boolean);
	 procedure SetExecuteApp(AValue: boolean);
	 procedure SetAppGlyph(AValue: boolean);
	 procedure SetAppName(AValue: string);
	 procedure SetSmallIcon(AValue: boolean);
	 procedure SetRunOnce(AValue: boolean);
	 procedure msCheckGlyph;
	 procedure LoadFileIcon;
	 procedure RunFile;
	 procedure BringApplicationToFront;
	 function FindApplicationWindowHandle: boolean;
	 function DoesWindowExist: boolean;
  public
	 constructor Create(AOwner: TComponent); override;
	 destructor Destroy; override;
	 procedure Click; override;
  published
	 property ExecuteBeforeOnClick: boolean read FExecuteFirst write SetExecuteFirst;
	 property ExecuteFile: boolean read FExecuteApp write SetExecuteApp;
	 property UseFileIcon: boolean read FAppGlyph write SetAppGlyph;
	 property FileName: string read FAppName write SetAppName;
	 property FileParams: string read FAppParams write FAppParams;
	 property WorkingDirectory: string read FWorkDir write FWorkDir;
	 property UseSmallIcon: boolean read FSmallIcon write SetSmallIcon;
	 property FileWindowState: TFileWindowState read FAppWindowState write FAppWindowState;
	 property RunOnce: boolean read FRunOnce write SetRunOnce;
  end;

  procedure Register;

implementation

uses
	Dialogs, SysUtils, Shellapi, Windows;

var
	giThreadWndHandle: hWnd;

constructor TbspRunAppBtn.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	FSmallIcon := True;
	FExecuteFirst := True;
	FExecuteApp := True;
	FAppName := '';
	FileParams := '';
	WorkingDirectory := '';
	FAppGlyph := False;
	FileWindowState := fwNormal;
	FFileHandle := 0;
	FThreadIdentifier := 0;
	FRunOnce := False;
end;

destructor TbspRunAppBtn.Destroy;
begin
	inherited Destroy;
end;

procedure TbspRunAppBtn.SetExecuteApp(AValue: boolean);
begin
	if (ExecuteBeforeOnClick = True) then
		FExecuteApp := True
	else
		FExecuteApp := AValue;
end;

procedure TbspRunAppBtn.SetExecuteFirst(AValue: boolean);
begin
	FExecuteFirst := AValue;
	if (AValue = True) then
		ExecuteFile := True;
end;

procedure TbspRunAppBtn.SetAppGlyph(AValue: boolean);
begin
	FAppGlyph := AValue;
	msCheckGlyph;
end;

procedure TbspRunAppBtn.SetAppName(AValue: string);
begin
	if (FAppName <> AValue) then
	begin
		FThreadIdentifier := 0;
		FFileHandle := 0;
	end;
	FAppName := Trim(AValue);
	msCheckGlyph;
end;

procedure TbspRunAppBtn.SetSmallIcon(AValue: boolean);
begin
	FSmallIcon := AValue;
	msCheckGlyph;
end;

procedure TbspRunAppBtn.SetRunOnce(AValue: boolean);
begin
	FRunOnce := AValue;
end;

procedure TbspRunAppBtn.msCheckGlyph;
begin
	if (UseFileIcon = True) and (FileName <> '') then
	begin
		if (FileExists(FileName) = True) then
		begin
			// reset glyphs
			Glyph.FreeImage;
			Glyph.ReleaseHandle;
			Glyph := nil;
			NumGlyphs := 1;
			LoadFileIcon;
		end;
	end;
end;

procedure TbspRunAppBtn.LoadFileIcon;
var
	SFI: TShFileInfo;
	lWidthHeight:	integer;
begin
	lWidthHeight := 32;
	FillChar(SFI, SizeOf(SFI), 0);
	if SHGetFileInfo(PCHAR(Filename),0, SFI, SizeOf(SFI), SHGFI_ICON + Ord(UseSmallIcon)) <> 0 then
	begin
		try
			if (UseSmallIcon = True) then lWidthHeight := 16;
			Glyph.FreeImage;
			Glyph.Width := lWidthHeight;
			Glyph.Height := lWidthHeight;
			DrawIconEx(Glyph.Canvas.Handle, 0,0, SFI.hIcon, lWidthHeight, lWidthHeight, 0, 0, DI_NORMAL);
		finally
		end;
	end;
end;

procedure TbspRunAppBtn.Click;
begin
	if (ExecuteBeforeOnClick = True) then
	begin
		RunFile;
		inherited Click;
	end
	else
	begin
		inherited Click;
		if (ExecuteFile = True) then
			RunFile;
	end;
end;

function EnumThrWnd(H : HWnd; Ignored : LongInt) : Bool; StdCall;
begin
	giThreadWndHandle := H;
	if (H <> 0) then
	begin
		Result := True;
		if (GetParent(H) = 0) then
		begin
			giThreadWndHandle := H;
			Result := False
		end;
	end
	else
		Result := True;
end;

procedure TbspRunAppBtn.RunFile;
const
  lWindowState: array[TFileWindowState] of Integer = (SW_SHOWNORMAL, SW_MINIMIZE, SW_SHOWMAXIMIZED);
var
	lStartupInfo:			TStartupInfo;
	lProcessInformation:	TProcessInformation;
	lStartApplication:		Boolean;
begin

	lStartApplication := True;
	if (FileName = '') then exit;

	// if run once is true check if you can find the application and bring it to front
	if (FFileHandle <> 0) and (RunOnce = True) then
	begin
		if (DoesWindowExist = True) then
		begin
			lStartApplication := False;
			BringApplicationToFront;
		end;
	end;

	// start the application
	if (lStartApplication = True) then
	begin
		if (WorkingDirectory = '') then
			WorkingDirectory := ExtractFileDir(FileName);
		// set defaults
		FillChar(lStartupInfo, SizeOf(lStartupInfo), 0);
		lStartupInfo.CB := SizeOf(lStartupInfo);
		lStartupInfo.lpTitle := PChar('test');
		// create process
		If CreateProcess(PChar(FileName),
		 PChar(FileParams),
		 NIL,
		 NIL,
		 False,
		 DETACHED_PROCESS OR NORMAL_PRIORITY_CLASS,
		 NIL,
		 PChar(WorkingDirectory),
		 lStartupInfo,
		 lProcessInformation)
		Then
		Begin
			// save the processidentifier for later checks
			FThreadIdentifier :=  lProcessInformation.dwThreadID;
			CloseHandle(lProcessInformation.hThread);
			// check for handle will be when used (because of time an app may need to start)
			WaitForInputIdle(lProcessInformation.hProcess, INFINITE); {let process start!}
			FindApplicationWindowHandle;
		end;
	end;
end;

function TbspRunAppBtn.DoesWindowExist: boolean;
var
	lClassName:				array [0..99] of Char;
	lCheckClass:			boolean;
begin
	FillChar(lClassName, SizeOf(lClassName), 0);
	lCheckClass := (GetClassName(FFileHandle, lClassName, SizeOf(lClassName)) > 0);
	Result := lCheckClass;
end;

function TbspRunAppBtn.FindApplicationWindowHandle: boolean;
var
	actwin,
	own:	hwnd;
begin
	try
		actwin := GetForegroundWindow();
		while True do
		begin
			own := GetWindow(actwin, GW_OWNER);
			if (own = 0) then
				break;
			actwin := own;
		end;
		FFileHandle := actwin;
		Result := True;
	except
		FFileHandle := 0;
		Result := False;
	end;
end;

procedure TbspRunAppBtn.BringApplicationToFront;
begin
	ShowWindow(FFileHandle, SW_RESTORE);
	SetForegroundWindow(FFileHandle);
end;

procedure Register;
begin
  RegisterComponents('bsp', [TbspRunAppBtn]);
end;


end.
